@@ -1,25 +1,24 @@
 /* regex.c: regular expression interface routines for the ed line editor. */
-/*  GNU ed - The GNU line editor.
-    Copyright (C) 1993, 1994 Andrew Moore, Talke Studio
-    Copyright (C) 2006-2017 Antonio Diaz Diaz.
+/* GNU ed - The GNU line editor.
+   Copyright (C) 1993, 1994 Andrew Moore, Talke Studio
+   Copyright (C) 2006-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stddef.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <regex.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,6 +27,8 @@
 #include "ed.h"
 
 
+static const char * const inv_pat_del = "Invalid pattern delimiter";
+static const char * const no_match = "No match";
 static regex_t * subst_regex_ = 0;	/* regex of previous substitution */
 
 static char * rbuf = 0;		/* replacement buffer */
@@ -114,8 +115,7 @@ static regex_t * get_compiled_regex( const char ** const ibufpp,
   const char delimiter = **ibufpp;
   int n;
 
-  if( delimiter == ' ' )
-    { set_error_msg( "Invalid pattern delimiter" ); return 0; }
+  if( delimiter == ' ' ) { set_error_msg( inv_pat_del ); return 0; }
   if( delimiter == '\n' || *++*ibufpp == delimiter ||
       ( **ibufpp == '\n' && !test_delimiter ) )
     {
@@ -129,7 +129,7 @@ static regex_t * get_compiled_regex( const char ** const ibufpp,
   /* exp compiled && not copied */
   if( exp && exp != subst_regex_ ) regfree( exp );
   else exp = ( &store[0] != subst_regex_ ) ? &store[0] : &store[1];
-  n = regcomp( exp, pat, 0 );
+  n = regcomp( exp, pat, extended_regexp() ? REG_EXTENDED : 0 );
   if( n )
     {
     char buf[80];
@@ -167,7 +167,7 @@ bool build_active_list( const char ** const ibufpp, const int first_addr,
   const char delimiter = **ibufpp;
 
   if( delimiter == ' ' || delimiter == '\n' )
-    { set_error_msg( "Invalid pattern delimiter" ); return false; }
+    { set_error_msg( inv_pat_del ); return false; }
   exp = get_compiled_regex( ibufpp, false );
   if( !exp ) return false;
   if( **ibufpp == delimiter ) ++*ibufpp;
@@ -205,12 +205,13 @@ int next_matching_node_addr( const char ** const ibufpp, const bool forward )
       }
     }
   while( addr != current_addr() );
-  set_error_msg( "No match" );
+  set_error_msg( no_match );
   return -1;
   }
 
 
-/* extract substitution replacement from the command buffer */
+/* Extract substitution replacement from the command buffer.
+   If isglobal, newlines in command-list are unescaped. */
 bool extract_replacement( const char ** const ibufpp, const bool isglobal )
   {
   static char * buf = 0;		/* temporary buffer */
@@ -221,7 +222,9 @@ bool extract_replacement( const char ** const ibufpp, const bool isglobal )
   if( delimiter == '\n' )
     { set_error_msg( "Missing pattern delimiter" ); return false; }
   ++*ibufpp;
-  if( **ibufpp == '%' && ( (*ibufpp)[1] == delimiter || (*ibufpp)[1] == '\n' ) )
+  if( **ibufpp == '%' &&		/* replacement is a single '%' */
+      ( (*ibufpp)[1] == delimiter ||
+        ( (*ibufpp)[1] == '\n' && ( !isglobal || (*ibufpp)[2] == 0 ) ) ) )
     {
     ++*ibufpp;
     if( !rbuf ) { set_error_msg( "No previous substitution" ); return false; }
@@ -229,12 +232,7 @@ bool extract_replacement( const char ** const ibufpp, const bool isglobal )
     }
   while( **ibufpp != delimiter )
     {
-    if( **ibufpp == '\n' )
-      {
-      if( isglobal && (*ibufpp)[1] != 0 )
-        { set_error_msg( "Invalid newline substitution" ); return false; }
-      break;
-      }
+    if( **ibufpp == '\n' && ( !isglobal || (*ibufpp)[1] == 0 ) ) break;
     if( !resize_buffer( &buf, &bufsz, i + 2 ) ) return false;
     if( ( buf[i++] = *(*ibufpp)++ ) == '\\' &&
         ( buf[i++] = *(*ibufpp)++ ) == '\n' && !isglobal )
@@ -264,28 +262,28 @@ static int replace_matched_text( char ** txtbufp, int * const txtbufszp,
                                  const regmatch_t * const rm, int offset,
                                  const int re_nsub )
   {
-  const char * sub = rbuf;
+  int i;
 
-  for( ; sub - rbuf < rlen; ++sub )
+  for( i = 0 ; i < rlen; ++i )
     {
     int n;
-    if( *sub == '&' )
+    if( rbuf[i] == '&' )
       {
       int j = rm[0].rm_so; int k = rm[0].rm_eo;
       if( !resize_buffer( txtbufp, txtbufszp, offset + k - j ) ) return -1;
       while( j < k ) (*txtbufp)[offset++] = txt[j++];
       }
-    else if( *sub == '\\' && *++sub >= '1' && *sub <= '9' &&
-             ( n = *sub - '0' ) <= re_nsub )
+    else if( rbuf[i] == '\\' && rbuf[++i] >= '1' && rbuf[i] <= '9' &&
+             ( n = rbuf[i] - '0' ) <= re_nsub )
       {
       int j = rm[n].rm_so; int k = rm[n].rm_eo;
       if( !resize_buffer( txtbufp, txtbufszp, offset + k - j ) ) return -1;
       while( j < k ) (*txtbufp)[offset++] = txt[j++];
       }
-    else
+    else		/* preceding 'if' skipped escaping backslashes */
       {
       if( !resize_buffer( txtbufp, txtbufszp, offset + 1 ) ) return -1;
-      (*txtbufp)[offset++] = *sub;
+      (*txtbufp)[offset++] = rbuf[i];
       }
     }
   if( !resize_buffer( txtbufp, txtbufszp, offset + 1 ) ) return -1;
@@ -313,6 +311,7 @@ static int line_replace( char ** txtbufp, int * const txtbufszp,
   if( !regexec( subst_regex_, txt, se_max, rm, 0 ) )
     {
     int matchno = 0;
+    bool infloop = false;
     do {
       if( global || snum == ++matchno )
         {
@@ -332,13 +331,14 @@ static int line_replace( char ** txtbufp, int * const txtbufszp,
         memcpy( *txtbufp + offset, txt, i ); offset += i;
         }
       txt += rm[0].rm_eo;
+      if( global && rm[0].rm_eo == 0 )
+        { if( !infloop ) infloop = true;	/* 's/^/#/g' is valid */
+          else { set_error_msg( "Infinite substitution loop" ); return -1; } }
       }
-    while( *txt && ( !changed || ( global && rm[0].rm_eo ) ) &&
+    while( *txt && ( !changed || global ) &&
            !regexec( subst_regex_, txt, se_max, rm, REG_NOTBOL ) );
     i = eot - txt;
     if( !resize_buffer( txtbufp, txtbufszp, offset + i + 2 ) ) return -1;
-    if( global && i > 0 && !rm[0].rm_eo )
-      { set_error_msg( "Infinite substitution loop" ); return -1; }
     if( isbinary() ) newline_to_nul( txt, i );
     memcpy( *txtbufp + offset, txt, i );		/* tail copy */
     memcpy( *txtbufp + offset + i, "\n", 2 );
@@ -389,6 +389,6 @@ bool search_and_replace( const int first_addr, const int second_addr,
       }
     }
   if( !match_found && !isglobal )
-    { set_error_msg( "No match" ); return false; }
+    { set_error_msg( no_match ); return false; }
   return true;
   }
